format shortEng
% clear all;
%load ('analysis.mat'); % must contain (at least) A, C and F matrices
I=eye(size(A));

%% Choice of method: 1=tiered hybrid LCA, 2=LCA, 3=IOA, 4=IO-part 
method=1;

%% Optional modifications of the A-matrix. Imported matrix from
%% analysis.mat is made for tiered hybrid LCA, so no modification needed.
if method==2
    A(72:329,1:71)=0; % To calculate a "pure" LCA
elseif method==3
    A(72:329,1:71)=A_bf_no0; % To calculate a "pure" IOA
    A(330:4426,1:71)=0;
elseif method==4
    A(330:4426,1:71)=0; % To calculate IO-part only
end

%% Finding total output
L=inv(I-A);
x=L*y;

%% Emissions and impacts (11 = number of y-vectors)
e=F*x; % Calculates the vector of total emissions generated for a given external demand
d=C*e; % Vector of total impacts for each scenario

%% Emissions distributed on foreground processes
temp=C*F*L;
D_pro_ff_w_ITD20=temp*diag(y(:,1));
D_pro_ff_w_ITDx0=temp*diag(y(:,2));
D_pro_ff_w_v05=temp*diag(y(:,3));
D_pro_ff_w_v06=temp*diag(y(:,4));
D_pro_ff_g_ITD20=temp*diag(y(:,5));
D_pro_ff_g_ITDx0=temp*diag(y(:,6));
D_pro_ff_g_v05=temp*diag(y(:,7));
D_pro_ff_g_v06=temp*diag(y(:,8));

% Make D_pro for emission sources
temp2=C*F;
D_pro_w_ITD20=temp2*diag(x(:,1))*D_em_class;
D_pro_w_ITDx0=temp2*diag(x(:,2))*D_em_class;
D_pro_w_v05=temp2*diag(x(:,3))*D_em_class;
D_pro_w_v06=temp2*diag(x(:,4))*D_em_class;
D_pro_g_ITD20=temp2*diag(x(:,5))*D_em_class;
D_pro_g_ITDx0=temp2*diag(x(:,6))*D_em_class;
D_pro_g_v05=temp2*diag(x(:,7))*D_em_class;
D_pro_g_v06=temp2*diag(x(:,8))*D_em_class;

D_pro_g=temp2*diag(x(:,7));


%% SF6 leakage CO2-eq contribution. NB! NOT be added to IO.
%% Is added to the contribution of offshore and onshore switchgear
if method==1||2
    char=22800; % ReCiPe's characterization factor of SF6 to CO2-eq
    leak=29.6; % leakage of CO2 for a GIS switchgear over entire lifetime, in kg 
    offshore_add=char*leak*y(16,1:8);
    onshore_add=char*leak*y(18,1:8);
    for j=1:8
        d(2,j)=d(2,j)+offshore_add(1,j)+onshore_add(1,j);
    end

    % To D_pro_ff
    D_pro_ff_w_ITD20(2,16)=D_pro_ff_w_ITD20(2,16)+offshore_add(1,1);
    D_pro_ff_w_ITD20(2,18)=D_pro_ff_w_ITD20(2,18)+onshore_add(1,1);
    D_pro_ff_w_ITDx0(2,16)=D_pro_ff_w_ITDx0(2,16)+offshore_add(1,2);
    D_pro_ff_w_ITDx0(2,18)=D_pro_ff_w_ITDx0(2,18)+onshore_add(1,2);
    D_pro_ff_w_v05(2,16)=D_pro_ff_w_v05(2,16)+offshore_add(1,3);
    D_pro_ff_w_v05(2,18)=D_pro_ff_w_v05(2,18)+onshore_add(1,3);
    D_pro_ff_w_v06(2,16)=D_pro_ff_w_v06(2,16)+offshore_add(1,4);
    D_pro_ff_w_v06(2,18)=D_pro_ff_w_v06(2,18)+onshore_add(1,4);
    D_pro_ff_g_ITD20(2,16)=D_pro_ff_g_ITD20(2,16)+offshore_add(1,5);
    D_pro_ff_g_ITD20(2,18)=D_pro_ff_g_ITD20(2,18)+onshore_add(1,5);
    D_pro_ff_g_ITDx0(2,16)=D_pro_ff_g_ITDx0(2,16)+offshore_add(1,6);
    D_pro_ff_g_ITDx0(2,18)=D_pro_ff_g_ITDx0(2,18)+onshore_add(1,6);
    D_pro_ff_g_v05(2,16)=D_pro_ff_g_v05(2,16)+offshore_add(1,7);
    D_pro_ff_g_v05(2,18)=D_pro_ff_g_v05(2,18)+onshore_add(1,7);
    D_pro_ff_g_v06(2,16)=D_pro_ff_g_v06(2,16)+offshore_add(1,8);
    D_pro_ff_g_v06(2,18)=D_pro_ff_g_v06(2,18)+onshore_add(1,8);

    % To D_pro
    D_pro_w_ITD20(2,7)=D_pro_w_ITD20(2,7)+onshore_add(1,1)+offshore_add(1,1);
    D_pro_w_ITDx0(2,7)=D_pro_w_ITDx0(2,7)+onshore_add(1,2)+offshore_add(1,2);
    D_pro_w_v05(2,7)=D_pro_w_v05(2,7)+onshore_add(1,3)+offshore_add(1,3);
    D_pro_w_v06(2,7)=D_pro_w_v06(2,7)+onshore_add(1,4)+offshore_add(1,4);
    D_pro_g_ITD20(2,7)=D_pro_g_ITD20(2,7)+onshore_add(1,5)+offshore_add(1,5);
    D_pro_g_ITDx0(2,7)=D_pro_g_ITDx0(2,7)+onshore_add(1,6)+offshore_add(1,6);
    D_pro_g_v05(2,7)=D_pro_g_v05(2,7)+onshore_add(1,7)+offshore_add(1,7);
    D_pro_g_v06(2,7)=D_pro_g_v06(2,7)+onshore_add(1,8)+offshore_add(1,8);
end

%% Calculates total impact divided by total energy produced for each
%% scenario
TWh=[4228 4732 7188 6951 4228 4732 7188 6951]; 
for i=1:18
    d_g(i,:)=d(i,:)./(TWh*1000*1000); %returns g CO2-eq/kWh
end

disp('analysis done');  


%% Taylor series expansion, n tiers
% 
% tiers=5;
% x_tay=zeros(size(y),tiers,11); % Matrix to store values from the taylor series expansion for each scenario
% d_tay=zeros(size(d),tiers,11); % Matrix to store values from the taylor series expansion for each scenario
% d_tay_acc=zeros(size(d),tiers,11); % Accumulated Taylor series impact
% d_tay_rel=zeros(size(d),tiers,11); % Relative accumulated contribution for each tier
% acc=0;
% 
% for i=0:tiers
%     %output=A^i*y;
%     impact=C*F*A^i*y;
%     acc=acc+impact;
%     for j=1:11
%         %x_tay(1:size(y),i+1,j)=output(1:size(y),j);
%         d_tay(1:size(d),i+1,j)=impact(1:size(d),j);
%         d_tay_acc(1:size(d),i+1,j)=acc(1:size(d),j);
%         d_tay_rel(1:size(d),i+1,j)=acc(1:size(d),j)./d(1:size(d),j);
%     end
% end
% 

% % E=zeros(p,m,n);
% D_pro=zeros(o,m,n);
% % D_str=zeros(o,p,n);
% % 
% for i=1:n
% %    E(1:p,1:m,i)=F*diag(x(1:m,i)); % Calculates the matrix of stressor intensities per unit output
%     D_pro(1:o,1:m,i)=C*E(1:p,1:m,i); % Matrix of impacts generated from each process for a given external demand
% %    D_str(1:o,1:p,i)=C*diag(e(1:p,i)); % Matrix of impacts generated from each stressor for a given external demand
% end

    
  
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    