% Matlab script start
load ('Databases.mat'); % Import of all necessary already established databases: 
% EXIOPOL (A_exiopol), Ecoinvent process matrix (A_Ecoinvent), Ecoinvent 
% stressor matrix (F), (modified to take into account EXIPOL stressors too)
% and the ReCiPe charachterization matrix (F).  

%% Choose method: 1=tiered hybrid LCA, 2=LCA, 3=IOA, 4=IO-part 
method=1;

%% Imports of foreground data 
A_f=xlsread('MatLab Import.xlsx', 'A_f', 'D6:BV4431'); % A_f contains 
% A_ff (foreground inter-process requirements matrix) and 
% A_nf (matrix of requirements from the foreground to Ecoinvent)
y_functional_unit=xlsread('MatLab Import.xlsx', 'y', 'E7:L77');
y_demand=xlsread('MatLab Import.xlsx', 'y_factors', 'D7:K77');
D_em_class=xlsread('MatLab Import.xlsx', 'P_class', 'C2:K4427');

A_f(isnan(A_f))=0; % Making sure all empty fields are 0
D_em_class(isnan(D_em_class))=0; % Making sure all empty fields are 0

% Create the A_pf (matrix of requirements from the foregtround 
% to EXIOPOL) matrices required for the different methods
A_pf_zero=xlsread('MatLab Import.xlsx', 'Abf_zero', 'D3:BV260');
A_pf_org=xlsread('MatLab Import.xlsx', 'Abf_org', 'D4:BV261');
A_pf_no0=A_exiopol*A_pf_org;
A_pf_with0=A_pf_no0.*A_pf_zero;

%% Construct A-matrix
[m n]=size(A_f);
A=zeros(m,m);
A(1:m,1:71)=A_f;
A(72:329,72:329)=A_exiopol;
A(330:m,330:m)=A_ecoinvent;

% Depending on method chosen, the A-matrix is adjusted accordingly:
if method==1
    A(72:329,1:71)=A_pf_with0; 
elseif method==2
    A(72:329,1:71)=0; % To calculate a "pure" LCA
elseif method==3
    A(72:329,1:71)=A_pf_no0; % To calculate a "pure" IOA
    A(330:4426,1:71)=0;
elseif method==4
    A(330:4426,1:71)=0; % To calculate IO-part only
end

%% Finding total output
I=eye(size(A));
L=inv(I-A);

y=zeros(4426,8);
y(1:71,1:8)=y_functional_unit.*y_demand;

x=L*y;

%% Emissions and impacts
e=F*x; % Calculates the vector of total emissions generated for a given external demand
d=C*e; % Vector of total impacts for each scenario

%% Emissions allocated to foreground processes
temp=C*F*L;
D_pro_ff_w_ITD20=temp*diag(y(:,1));
D_pro_ff_w_ITDx0=temp*diag(y(:,2));
D_pro_ff_w_v05=temp*diag(y(:,3));
D_pro_ff_w_v06=temp*diag(y(:,4));
D_pro_ff_g_ITD20=temp*diag(y(:,5));
D_pro_ff_g_ITDx0=temp*diag(y(:,6));
D_pro_ff_g_v05=temp*diag(y(:,7));
D_pro_ff_g_v06=temp*diag(y(:,8));

% Emissions allocated to emssision sources
temp2=C*F;
D_pro_w_ITD20=temp2*diag(x(:,1))*D_em_class;
D_pro_w_ITDx0=temp2*diag(x(:,2))*D_em_class;
D_pro_w_v05=temp2*diag(x(:,3))*D_em_class;
D_pro_w_v06=temp2*diag(x(:,4))*D_em_class;
D_pro_g_ITD20=temp2*diag(x(:,5))*D_em_class;
D_pro_g_ITDx0=temp2*diag(x(:,6))*D_em_class;
D_pro_g_v05=temp2*diag(x(:,7))*D_em_class;
D_pro_g_v06=temp2*diag(x(:,8))*D_em_class;


%% SF6 leakage CO2-eq contribution. Not added to IOA or IO-part.
%% Is added to the contribution of offshore and onshore switchgear
if method==1||2
    char=22800; % ReCiPe's characterization factor of SF6 to CO2-eq
    leak=29.6; % leakage of CO2 for a GIS switchgear over entire lifetime, in kg 
    offshore_add=char*leak*y(16,1:8);
    onshore_add=char*leak*y(18,1:8);
    for j=1:8
        d(2,j)=d(2,j)+offshore_add(1,j)+onshore_add(1,j);
    end

    % To D_pro_ff
    D_pro_ff_w_ITD20(2,16)=D_pro_ff_w_ITD20(2,16)+offshore_add(1,1);
    D_pro_ff_w_ITD20(2,18)=D_pro_ff_w_ITD20(2,18)+onshore_add(1,1);
    D_pro_ff_w_ITDx0(2,16)=D_pro_ff_w_ITDx0(2,16)+offshore_add(1,2);
    D_pro_ff_w_ITDx0(2,18)=D_pro_ff_w_ITDx0(2,18)+onshore_add(1,2);
    D_pro_ff_w_v05(2,16)=D_pro_ff_w_v05(2,16)+offshore_add(1,3);
    D_pro_ff_w_v05(2,18)=D_pro_ff_w_v05(2,18)+onshore_add(1,3);
    D_pro_ff_w_v06(2,16)=D_pro_ff_w_v06(2,16)+offshore_add(1,4);
    D_pro_ff_w_v06(2,18)=D_pro_ff_w_v06(2,18)+onshore_add(1,4);
    D_pro_ff_g_ITD20(2,16)=D_pro_ff_g_ITD20(2,16)+offshore_add(1,5);
    D_pro_ff_g_ITD20(2,18)=D_pro_ff_g_ITD20(2,18)+onshore_add(1,5);
    D_pro_ff_g_ITDx0(2,16)=D_pro_ff_g_ITDx0(2,16)+offshore_add(1,6);
    D_pro_ff_g_ITDx0(2,18)=D_pro_ff_g_ITDx0(2,18)+onshore_add(1,6);
    D_pro_ff_g_v05(2,16)=D_pro_ff_g_v05(2,16)+offshore_add(1,7);
    D_pro_ff_g_v05(2,18)=D_pro_ff_g_v05(2,18)+onshore_add(1,7);
    D_pro_ff_g_v06(2,16)=D_pro_ff_g_v06(2,16)+offshore_add(1,8);
    D_pro_ff_g_v06(2,18)=D_pro_ff_g_v06(2,18)+onshore_add(1,8);

    % To D_pro
    D_pro_w_ITD20(2,7)=D_pro_w_ITD20(2,7)+onshore_add(1,1)+offshore_add(1,1);
    D_pro_w_ITDx0(2,7)=D_pro_w_ITDx0(2,7)+onshore_add(1,2)+offshore_add(1,2);
    D_pro_w_v05(2,7)=D_pro_w_v05(2,7)+onshore_add(1,3)+offshore_add(1,3);
    D_pro_w_v06(2,7)=D_pro_w_v06(2,7)+onshore_add(1,4)+offshore_add(1,4);
    D_pro_g_ITD20(2,7)=D_pro_g_ITD20(2,7)+onshore_add(1,5)+offshore_add(1,5);
    D_pro_g_ITDx0(2,7)=D_pro_g_ITDx0(2,7)+onshore_add(1,6)+offshore_add(1,6);
    D_pro_g_v05(2,7)=D_pro_g_v05(2,7)+onshore_add(1,7)+offshore_add(1,7);
    D_pro_g_v06(2,7)=D_pro_g_v06(2,7)+onshore_add(1,8)+offshore_add(1,8);
end

%% Calculates total impact divided by total energy produced for each
%% scenario
TWh=[4228 4732 7188 6951 4228 4732 7188 6951]; 
for i=1:18
    d_g(i,:)=d(i,:)./(TWh*1000*1000); %returns g CO2-eq/kWh
end

disp('analysis done');
% Matlab script end